(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *
 * FWTesting.thy ---
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2010 Université Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************.***************************************************************)
(* $Id: Normalized.thy 9105 2011-01-25 22:03:54Z makarius $ *)

theory 
  "Normalized"
imports
  "../../PolicyDefinitions"
begin


definition Policy :: "(IntegerPort net,port) Combinators" where
"Policy = 
DenyAll \<oplus>  AllowPortFromTo one two 199 \<oplus> AllowPortFromTo one two 38 \<oplus> AllowPortFromTo one three 533 \<oplus> DenyAllFromTo two one\<oplus> AllowPortFromTo three two 578 \<oplus> AllowPortFromTo two one 645 \<oplus> AllowPortFromTo one three 694 \<oplus> AllowPortFromTo three one 871
"


lemma "dupl (policy2list Policy) = X"
apply (simp add: Policy_def)
oops



lemmas policies =  Policy_def 

lemmas UnfoldNetworkAndPolicy = UnfoldPolicy PLemmas policies


lemmas normalizeUnfold = makeFUT_def normalize_def policies Nets_List_def bothNets_def aux aux2 bothNets_def

lemmas casesSimp = fixDefs packet_Nets_def PLemmas UnfoldNetworkAndPolicy


lemma noMT2: "\<forall> x \<in> set (policy2list Policy). dom (C x) \<noteq> {}"
apply (simp add: UnfoldNetworkAndPolicy)
done


lemma count_the_rules: "(int (length(policy2list (list2policy(normalize Policy)))) = post) \<and> (int(length (policy2list Policy)) = pre) \<and>
 (int (length((normalize Policy))) = Partitions)"
apply (insert nets_different noMT2)
apply (simp add:  normalizeUnfold)
apply (thin_tac "?X")+
oops


lemma normalize: "normalize Policy = X"
apply (insert nets_different noMT2)
apply (simp add: normalizeUnfold, thin_tac "?X",thin_tac "?S")
oops


test_spec "fixElements x \<longrightarrow> makeFUT FUT Policy x 0"
apply (insert nets_different noMT2)
apply (simp add: normalizeUnfold , thin_tac "?X",thin_tac "?S")
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)
apply (simp add: casesSimp)
apply (gen_test_cases "FUT")
store_test_thm "foo0"

gen_test_data "foo0"

thm foo0.test_data


test_spec "fixElements x \<longrightarrow> makeFUT FUT Policy x 1"
apply (insert nets_different noMT2)
apply (simp add: normalizeUnfold , thin_tac "?X",thin_tac "?S")
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)
apply (simp add: casesSimp)
apply (gen_test_cases "FUT")
store_test_thm "foo1"


test_spec "fixElements x \<longrightarrow> makeFUT FUT Policy x 2"
apply (insert nets_different noMT2)
apply (simp add: normalizeUnfold , thin_tac "?X",thin_tac "?S")
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)
apply (simp add: casesSimp)
apply (gen_test_cases "FUT")
store_test_thm "foo2"



test_spec "fixElements x \<longrightarrow>  FUT x = C Policy x"
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)
apply (simp add: fixDefs packet_Nets_def)
apply (simp add: PLemmas  UnfoldNetworkAndPolicy)
apply (gen_test_cases "FUT")
apply simp_all
store_test_thm "full"
gen_test_data "full"

export_test_data "full.data" "full"






consts protect :: "'a \<Rightarrow> 'b \<Rightarrow> 'a"
defs protect_def : "protect x a == x"

lemma protectI: "protect x a \<Longrightarrow> x"
apply (simp add: protect_def)
done


lemma normalize: "normalize Policy =  [DenyAll,
     DenyAllFromTo three two \<oplus> DenyAllFromTo two three \<oplus> AllowPortFromTo three two 578,
     DenyAllFromTo one two \<oplus>
     ((DenyAllFromTo two one \<oplus> AllowPortFromTo one two 199) \<oplus> AllowPortFromTo one two 38) \<oplus>
     AllowPortFromTo two one 645,
     DenyAllFromTo one three \<oplus>
     DenyAllFromTo three one \<oplus>
     (AllowPortFromTo one three 533 \<oplus> AllowPortFromTo one three 694) \<oplus>
     AllowPortFromTo three one 871]"
apply (insert nets_different noMT2)
apply (simp add: normalizeUnfold)
done

lemma disj: "disjDomGD (Comb2PolList (rev (normalize Policy)))"
apply (simp add: normalize)
apply (simp add: disjDomGD_def C.simps)
apply (simp add: PLemmas UnfoldNetworkAndPolicy)
apply auto
done


lemma PolUniv: "dom (C Policy) = UNIV"
apply (simp add: Policy_def)
apply (simp add: PLemmas UnfoldNetworkAndPolicy)
done

lemma N2: "DenyAll \<in> set (policy2list Policy)"
apply (simp add: UnfoldNetworkAndPolicy)
done

lemma N3: "allNetsDistinct (policy2list Policy)"
apply (simp add: UnfoldNetworkAndPolicy)
sorry

lemma N4: "all_in_list (policy2list Policy) (Nets_List Policy)"
apply (simp add: Policy_def)
apply (simp add: PLemmas UnfoldNetworkAndPolicy)
apply auto
sorry


lemma A1: " list2policyP (Comb2PolList (rev (normalize Policy))) (a, (aa, b), (ab, ba), bb) =
       C Policy (a, (aa, b), (ab, ba), bb)"
sorry

lemma A2: "disjDomGD (Comb2PolList (rev (normalize Policy)))"
sorry


lemma wrong: "  fixElements (a, (aa, b), (ab, ba), bb) \<Longrightarrow>
       (a, (aa, b), (ab, ba), bb)
       \<notin> dom (C (DenyAllFromTo one three \<oplus>
                 DenyAllFromTo three one \<oplus>
                 (AllowPortFromTo one three 533 \<oplus> AllowPortFromTo one three 694) \<oplus>
                 AllowPortFromTo three one 871)) \<and>
       (a, (aa, b), (ab, ba), bb)
       \<notin> dom (C (DenyAllFromTo one two \<oplus>
                 ((DenyAllFromTo two one \<oplus> AllowPortFromTo one two 199) \<oplus>
                  AllowPortFromTo one two 38) \<oplus>
                 AllowPortFromTo two one 645)) \<and>
       (a, (aa, b), (ab, ba), bb)
       \<notin> dom (C (DenyAllFromTo three two \<oplus>
                 DenyAllFromTo two three \<oplus> AllowPortFromTo three two 578)) \<longrightarrow>
       FUT (a, (aa, b), (ab, ba), bb) = C DenyAll (a, (aa, b), (ab, ba), bb)
 "
sorry


test_spec "fixElements x \<longrightarrow>  FUT x = C Policy x"
apply (prepare_fw_spec) 
apply (rule_tac PL = "(Comb2PolList (rev (normalize Policy)))" in distrFUTLG1)
apply (simp add: normalize)
apply (simp add: PolUniv)
apply (rule A1)
apply (rule A2)
apply (simp add: normalize)
apply (simp add: FUTListGD_def gatherDomain_def)
apply (rule conjI)
defer 1 
apply (rule conjI)
apply (simp add: casesSimp)
defer 1
apply (rule conjI)
apply (simp add: casesSimp)
defer 1
apply (simp add: casesSimp)
apply (gen_test_cases "FUT")
apply (simp_all add: casesSimp)
store_test_thm "fullNorm"

declare casesSimp[iff]

gen_test_data "fullNorm"

thm fullNorm.test_data
thm fullNorm.pos





test_spec "fixElements x \<longrightarrow>  FUT x = C Policy x"
apply (prepare_fw_spec) 
apply (rule_tac PL = "(Comb2PolList (rev (normalize Policy)))" in distrFUTLG1)
apply (simp add: normalize)
apply (simp add: PolUniv)
apply (rule A1)
apply (rule A2)
apply (simp add: normalize)
apply (simp add: FUTListGD_def gatherDomain_def)
apply (simp_all add: fixElements_def id_def)
apply (thin_tac "?X")
apply (rule conjI) 
defer 1
apply (rule conjI)
defer 1
apply (rule conjI)
apply (simp_all add: casesSimp)
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (gen_test_cases "FUT")
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (simp add: protect_def)
apply (gen_test_cases FUT)
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (simp add: protect_def)
apply (gen_test_cases FUT)
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (rule_tac a = FUT in protectI)
defer 1
apply (simp add: protect_def)
apply simp_all
apply (gen_test_cases FUT)
apply (simp_all add: protect_def)


store_test_thm "fullNormProtect"



end
